@@ -19,7 +19,7 @@ _ECR_DELETION_TIMEOUT = datetime.timedelta(days=2)
 def _get_meadowrun_task_queue_urls(sqs_client: Any) -> Iterable[str]:
     """Gets all SQS queue URLs created for meadowrun tasks"""
     for page in sqs_client.get_paginator("list_queues").paginate(
-        QueueNamePrefix="meadowrunTask"
+        QueueNamePrefix="meadowrun-task"
     ):
         if "QueueUrls" in page:
             for queue_url in page["QueueUrls"]:
@@ -34,15 +34,21 @@ def delete_old_task_queues(region_name: str) -> None:
     now = datetime.datetime.utcnow()
     client = boto3.client("sqs", region_name=region_name)
     for queue_url in _get_meadowrun_task_queue_urls(client):
-        response = client.get_queue_attributes(
-            QueueUrl=queue_url,
-            AttributeNames=[
-                "ApproximateNumberOfMessages",
-                "ApproximateNumberOfMessagesNotVisible",
-                "CreatedTimestamp",
-                "LastModifiedTimestamp",
-            ],
+        success, response = ignore_boto3_error_code(
+            lambda: client.get_queue_attributes(
+                QueueUrl=queue_url,
+                AttributeNames=[
+                    "ApproximateNumberOfMessages",
+                    "ApproximateNumberOfMessagesNotVisible",
+                    "CreatedTimestamp",
+                    "LastModifiedTimestamp",
+                ],
+            ),
+            "AWS.SimpleQueueService.NonExistentQueue",
         )
+        if not success:
+            continue
+        assert response is not None  # just for mypy
         attributes = response["Attributes"]
         # we should only need to check LastModifiedTimestamp, but we include
         # CreatedTimestamp just in case
@@ -51,8 +57,7 @@ def delete_old_task_queues(region_name: str) -> None:
             datetime.datetime.fromtimestamp(int(attributes["LastModifiedTimestamp"])),
         )
         if (
-            attributes["ApproximateNumberOfMessages"] == "0"
-            and attributes["ApproximateNumberOfMessagesNotVisible"] == "0"
+            attributes["ApproximateNumberOfMessagesNotVisible"] == "0"
             and now - last_modified > _QUEUE_DELETION_TIMEOUT
         ):
             print(f"Deleting queue {queue_url}, was last modified at {last_modified}")
@@ -60,10 +65,13 @@ def delete_old_task_queues(region_name: str) -> None:
 
 
 def delete_all_task_queues(region_name: str) -> None:
-    """WARNING this will causing running run_map jobs to fail"""
+    """WARNING this will cause running run_map jobs to fail"""
     client = boto3.client("sqs", region_name=region_name)
     for queue_url in _get_meadowrun_task_queue_urls(client):
-        client.delete_queue(QueueUrl=queue_url)
+        ignore_boto3_error_code(
+            lambda: client.delete_queue(QueueUrl=queue_url),
+            "AWS.SimpleQueueService.NonExistentQueue",
+        )
 
 
 def delete_unused_images(region_name: str) -> None:
