@@ -103,16 +103,20 @@ class Deployment:
         conda_env: Optional[str] = None,
         environment_variables: Optional[Dict[str, str]] = None,
     ) -> Deployment:
-        """A deployment that mirrors local environment and code.
+        """A deployment that mirrors the local environment and code.
 
-        :param include_sys_path: if True, find paths on sys.path, copy them to
-            the remote machines, and add them to sys.path there. Ignores any installed
-            packages. Defaults to True.
-        :param additional_paths: local code paths to mirror on the remote servers.
-        :param conda_env: name or full path of locally installed conda environment to
-            use. Defaults to None which means the currently activated env.
-        :param environment_variables: e.g. :code:`{"PYTHONHASHSEED": "0"}`. These
-            environment variables will be set in the remote environment.
+        Args:
+            include_sys_path: if True, find python code in the paths in sys.path
+                (effectively "your local code"), copies them to the remote machines, and
+                add them to sys.path there. Ignores any installed packages.
+            additional_paths: local code paths to copy to the remote machine.
+            conda_env: name or full path of locally installed conda environment to
+                use. Defaults to None which means the currently activated env.
+            environment_variables: e.g. `{"PYTHONHASHSEED": "0"}`. These environment
+                variables will be set in the remote environment.
+
+        Returns:
+            A `Deployment` object that can be passed to the `run_*` functions.
         """
         if platform.system() != "Linux":
             raise ValueError(
