@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import asyncio
 import asyncio.subprocess
 import dataclasses
@@ -5,6 +7,7 @@ import itertools
 import os
 import os.path
 import pathlib
+import pickle
 import shutil
 import sys
 import traceback
@@ -14,18 +17,15 @@ from typing import (
     Dict,
     Iterable,
     List,
-    Literal,
     Optional,
     Sequence,
     Tuple,
 )
 
-from meadowrun._vendor.aiodocker import containers as aiodocker_containers
+from typing_extensions import Literal
 
-from meadowrun.deployment_manager import (
-    compile_environment_spec_to_container,
-    get_code_paths,
-)
+from meadowrun._vendor import aiodocker
+from meadowrun._vendor.aiodocker import containers as aiodocker_containers
 from meadowrun.config import (
     MEADOWRUN_AGENT_PID,
     MEADOWRUN_CODE_MOUNT_LINUX,
@@ -38,6 +38,10 @@ from meadowrun.credentials import (
     get_docker_credentials,
     get_matching_credentials,
 )
+from meadowrun.deployment_manager import (
+    compile_environment_spec_to_container,
+    get_code_paths,
+)
 from meadowrun.docker_controller import (
     run_container,
     get_image_environment_variables,
@@ -50,6 +54,12 @@ from meadowrun.meadowrun_pb2 import (
     PyFunctionJob,
     StringPair,
 )
+from meadowrun.run_job_core import (
+    CloudProviderType,
+    Host,
+    JobCompletion,
+    MeadowrunException,
+)
 from meadowrun.shared import pickle_exception
 
 ProcessStateEnum = ProcessState.ProcessStateEnum
@@ -357,14 +367,13 @@ async def _launch_non_container_job(
         f"PYTHONPATH={env_vars['PYTHONPATH']}; log_file_name={log_file_name}"
     )
 
-    with open(log_file_name, "w", encoding="utf-8") as log_file:
-        process = await asyncio.subprocess.create_subprocess_exec(
-            *job_spec_transformed.command_line,
-            stdout=log_file,
-            stderr=asyncio.subprocess.STDOUT,
-            cwd=working_directory,
-            env=env_vars,
-        )
+    process = await asyncio.subprocess.create_subprocess_exec(
+        *job_spec_transformed.command_line,
+        stdout=asyncio.subprocess.PIPE,
+        stderr=asyncio.subprocess.STDOUT,
+        cwd=working_directory,
+        env=env_vars,
+    )
 
     # (5) return the pid and continuation
     return process.pid, _non_container_job_continuation(
@@ -394,6 +403,11 @@ async def _non_container_job_continuation(
     try:
         # wait for the process to finish
         # TODO add an optional timeout
+
+        with open(log_file_name, "wb") as log_file:
+            async for line in process.stdout:  # type: ignore
+                log_file.write(line)
+                sys.stdout.buffer.write(line)
         returncode = await process.wait()
         return _completed_job_state(
             job_spec_type,
@@ -485,7 +499,7 @@ async def _launch_container_job(
         f"log_file_name={log_file_name}"
     )
 
-    container = await run_container(
+    container, docker_client = await run_container(
         container_image_name,
         # json serializer needs a real list, not a protobuf fake list
         job_spec_transformed.command_line,
@@ -494,6 +508,7 @@ async def _launch_container_job(
     )
     return container.id, _container_job_continuation(
         container,
+        docker_client,
         job_spec_type,
         job.job_id,
         io_folder,
@@ -504,6 +519,7 @@ async def _launch_container_job(
 
 async def _container_job_continuation(
     container: aiodocker_containers.DockerContainer,
+    docker_client: aiodocker.Docker,
     job_spec_type: Literal["py_command", "py_function"],
     job_id: str,
     io_folder: str,
@@ -513,7 +529,9 @@ async def _container_job_continuation(
     """
     Writes the container's logs to log_file_name, waits for the container to finish, and
     then returns a ProcessState indicating the state of this container when it
-    finished
+    finished.
+
+    docker_client just needs to be closed when the container process has completed.
     """
     try:
         # Docker appears to have an objection to having a log driver that can produce
@@ -523,6 +541,7 @@ async def _container_job_continuation(
         #  own plain text/whatever log driver for docker.
         with open(log_file_name, "w", encoding="utf-8") as f:
             async for line in container.log(stdout=True, stderr=True, follow=True):
+                print(line, end="")
                 f.write(line)
 
         wait_result = await container.wait()
@@ -549,6 +568,8 @@ async def _container_job_continuation(
             state=ProcessStateEnum.ERROR_GETTING_STATE,
             pickled_result=pickle_exception(e, result_highest_pickle_protocol),
         )
+    finally:
+        await docker_client.__aexit__(None, None, None)
 
 
 def _completed_job_state(
@@ -629,7 +650,7 @@ def _get_default_working_folder() -> str:
 
 def _set_up_working_folder(
     working_folder: Optional[str],
-) -> Tuple[str, str, str, str]:
+) -> Tuple[str, str, str, str, str]:
     """
     Sets the working_folder to a default if it's not set, creates the necessary
     subfolders, gets a machine-wide lock on the working folder, then returns io_folder,
@@ -651,13 +672,22 @@ def _set_up_working_folder(
     git_repos_folder = os.path.join(working_folder, "git_repos")
     # see CodeDeploymentManager
     local_copies_folder = os.path.join(working_folder, "local_copies")
+    # misc folder for e.g. storing environment export files sent from local machine
+    misc_folder = os.path.join(working_folder, "misc")
 
     os.makedirs(io_folder, exist_ok=True)
     os.makedirs(job_logs_folder, exist_ok=True)
     os.makedirs(git_repos_folder, exist_ok=True)
     os.makedirs(local_copies_folder, exist_ok=True)
+    os.makedirs(misc_folder, exist_ok=True)
 
-    return io_folder, job_logs_folder, git_repos_folder, local_copies_folder
+    return (
+        io_folder,
+        job_logs_folder,
+        git_repos_folder,
+        local_copies_folder,
+        misc_folder,
+    )
 
 
 async def _get_credentials_for_job(
@@ -724,7 +754,9 @@ async def _get_credentials_for_job(
 
 
 async def run_local(
-    job: Job, working_folder: Optional[str] = None
+    job: Job,
+    working_folder: Optional[str] = None,
+    cloud: Optional[Tuple[CloudProviderType, str]] = None,
 ) -> Tuple[ProcessState, Optional[asyncio.Task[ProcessState]]]:
     """
     Runs a job locally using the specified working_folder (or uses the default). Meant
@@ -749,6 +781,7 @@ async def run_local(
         job_logs_folder,
         git_repos_folder,
         local_copies_folder,
+        misc_folder,
     ) = _set_up_working_folder(working_folder)
 
     # unpickle credentials if necessary
@@ -759,7 +792,7 @@ async def run_local(
 
     try:
         # first, get the code paths
-        code_paths = await get_code_paths(
+        code_paths, interpreter_spec_path = await get_code_paths(
             git_repos_folder, local_copies_folder, job, code_deployment_credentials
         )
 
@@ -768,9 +801,23 @@ async def run_local(
         interpreter_deployment = job.WhichOneof("interpreter_deployment")
 
         if interpreter_deployment == "environment_spec_in_code":
+            if interpreter_spec_path is None:
+                raise ValueError(
+                    "Cannot specify environment_spec_in_code and not provide any code "
+                    "paths"
+                )
             job.server_available_container.CopyFrom(
                 await compile_environment_spec_to_container(
-                    job.environment_spec_in_code, code_paths
+                    job.environment_spec_in_code, interpreter_spec_path, cloud
+                )
+            )
+            interpreter_deployment = "server_available_container"
+
+        if interpreter_deployment == "environment_spec":
+
+            job.server_available_container.CopyFrom(
+                await compile_environment_spec_to_container(
+                    job.environment_spec, misc_folder, cloud
                 )
             )
             interpreter_deployment = "server_available_container"
@@ -804,6 +851,10 @@ async def run_local(
                 os.getpid()
             )
 
+        # add PYTHONUNBUFFERED=1, but don't modify if it already exists
+        if "PYTHONUNBUFFERED" not in job_spec_transformed.environment_variables:
+            job_spec_transformed.environment_variables["PYTHONUNBUFFERED"] = "1"
+
         # Merge in the user specified environment, variables, these should always take
         # precedence. A little sloppy to modify in place, but should be fine
         # TODO consider warning if we're overwriting any variables that already exist
@@ -887,3 +938,35 @@ async def run_local(
             ),
             None,
         )
+
+
+@dataclasses.dataclass(frozen=True)
+class LocalHost(Host):
+    async def run_job(self, job: Job) -> JobCompletion[Any]:
+        initial_update, continuation = await run_local(job)
+        if (
+            initial_update.state != ProcessState.ProcessStateEnum.RUNNING
+            or continuation is None
+        ):
+            result = initial_update
+        else:
+            result = await continuation
+
+        if result.state == ProcessState.ProcessStateEnum.SUCCEEDED:
+            job_spec_type = job.WhichOneof("job_spec")
+            # we must have a result from functions, in other cases we can optionally
+            # have a result
+            if job_spec_type == "py_function" or result.pickled_result:
+                unpickled_result = pickle.loads(result.pickled_result)
+            else:
+                unpickled_result = None
+
+            return JobCompletion(
+                unpickled_result,
+                result.state,
+                result.log_file_name,
+                result.return_code,
+                "localhost",
+            )
+        else:
+            raise MeadowrunException(result)
